package com.eldos.sp;

import SecureBlackbox.Base.*;
import SecureBlackbox.SAMLCommon.*;
import SecureBlackbox.XML.SBXMLDefs;
import SecureBlackbox.XML.TElXMLDOMDocument;
import SecureBlackbox.XML.TElXMLUTF8Codec;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.io.PrintWriter;
import java.security.InvalidParameterException;
import java.util.Date;

public class ServiceProviderServlet extends HttpServlet {

    private String spToIDPBinding, idpToSPBinding, idpSSOURL, acsURL, spIssuer, signCertPath;
    private boolean signRequests;

    @Override
    public void init(ServletConfig config) throws ServletException {
        super.init(config);

        spToIDPBinding = config.getInitParameter("sp_to_idp_binding");
        idpToSPBinding = config.getInitParameter("idp_to_sp_binding");
        idpSSOURL = config.getInitParameter("idp_sso_url");
        acsURL = config.getInitParameter("sp_acs_url");
        spIssuer = config.getInitParameter("sp_issuer");
        signCertPath = config.getInitParameter("sign_cert");

        String s = config.getInitParameter("sign_requests");
        signRequests = Boolean.parseBoolean(s);
    }

    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        HttpSession session = req.getSession(true);
        boolean authorized = false;

        Object o = session.getAttribute("auth");
        if (o != null)
            authorized = (Boolean) o;

        if (authorized) {
            RequestDispatcher view = req.getRequestDispatcher("/WEB-INF" + req.getRequestURI());
            view.forward(req, resp);
            return;
        }

        // generate SAML request to IdP
        TElSAMLAuthnRequestElement samlReq = new TElSAMLAuthnRequestElement();
        TElX509Certificate cert = null;

        samlReq.setID(SBRandom.sbRndString(20));
        // save request ID to check it later
        session.setAttribute("samlReqID", samlReq.getID());

        samlReq.setIssueInstant(SBUtils.utcNow());
        samlReq.setDestination(idpSSOURL);
        samlReq.setAssertionConsumerServiceURL(acsURL);
        samlReq.setProtocolBinding(idpToSPBinding);

        samlReq.setIssuer(new TElSAMLIssuerElement());
        samlReq.getIssuer().setValue(spIssuer);

        samlReq.setNameIDPolicy(new TElSAMLNameIDPolicyElement());
        samlReq.getNameIDPolicy().setAllowCreate(true);
        samlReq.getNameIDPolicy().setUseAllowCreate(true);
        samlReq.getNameIDPolicy().setFormat(SBXMLSAMLCore.xmlSAMLNameIDFormatTransient);

        if (signRequests)
        {
            cert = new TElX509Certificate();
            cert.loadFromFileAuto(signCertPath, "");

            TElSAMLSignatureHandler sigHandler = new TElSAMLSignatureHandler();
            sigHandler.setCertificate(cert);

            samlReq.setHandler(sigHandler);
            samlReq.setSigned(true);
        }

        TElXMLDOMDocument doc = new TElXMLDOMDocument();
        doc.appendChild(samlReq.saveToXML(doc));

        if (SBXMLSAMLBind.samlBindingRedirect.equals(spToIDPBinding)) {
            // Redirect binding
            TElSAMLRedirectBinding binding = new TElSAMLRedirectBinding();
            binding.setURL(idpSSOURL);
            binding.setRelayState(req.getRequestURL().toString()); // save request URL
            binding.setSignatureAlg(SBXMLDefs.xmlSignatureAlgorithmRSASHA1);

            if (signRequests)
                binding.setKeyMaterial(cert.getKeyMaterial());

            String bindingRes = binding.saveRequest(doc);

            resp.setStatus(302); // redirect
            resp.setHeader("Location", bindingRes);
        } else if (SBXMLSAMLBind.samlBindingPOST.equals(spToIDPBinding)) {
            // POST binding
            TElSAMLPOSTBinding binding = new TElSAMLPOSTBinding();
            binding.setURL(idpSSOURL);
            binding.setRelayState(req.getRequestURL().toString()); // save request URL
            binding.setMode(TSBSAMLPOSTBindingMode.spmServer);

            String bindingRes = binding.saveRequest(doc);

            resp.setStatus(200); // OK
            resp.getWriter().println(bindingRes);
        } else
            throw new InvalidParameterException();
    }
}
