import java.util.Vector;
import javax.swing.table.*;
import SecureBlackbox.Base.*;
import SecureBlackbox.ZIP.*;

public class KeyTableModel extends AbstractTableModel {
	private static final long serialVersionUID = -4060741556725801933L;
	public static final int FILENAME_INDEX   = 0;
	public static final int CSIZE_INDEX = 1;
    public static final int SIZE_INDEX  = 2;
    public static final int CALG_INDEX = 3;
    public static final int VERS_INDEX = 4;
    public static final int SEC_INDEX = 5;
    
    protected String[] columnNames = {
    		"File name", 
    		"Compressed size", 
    		"Original size", 
    		"Compression algorithm",
    		"Version to extract",
    		"Security" };
    
    protected Vector<KeyDataRow> dataVector = new Vector<KeyDataRow>();

 	public KeyTableModel(TElZipWriter elZipWriter, String currentPath) {
 		TElZipArchiveDirectoryEntry rootEntry = elZipWriter.getDirectory();

		if (currentPath.length() > 0)
		{
			rootEntry = rootEntry.entryWithPath(currentPath);
			if (rootEntry == null)
				return;
		}
		
		if (currentPath.length() > 0)
		{
			dataVector.add(new KeyDataRow("..", null));
		}

		for (int i = 0; i < rootEntry.getEntriesCount(); i++)
		{
			TElZipArchiveDirectoryEntry entry = rootEntry.getEntry(i);
			String name = entry.getFileName(); 
			if (entry.getIsDirectory())
			{
				String m = SBZipUtils.zipCompressionMethodName(entry.getCompressionMethod());
				String v = SBZipUtils.zipVersionStr(entry.getVersionToExtract());
				dataVector.add(new KeyDataRow(name, 0, 0, m, v, "", entry));
			} 
			else
			{
				String m = SBZipUtils.zipCompressionMethodName(entry.getCompressionMethod());
				String v = SBZipUtils.zipVersionStr(entry.getVersionToExtract());
				long csize = entry.getCompressedSize();
                String st = getSecurityString(entry);
				dataVector.add(new KeyDataRow(name, csize, entry.getFileSize(), m, v, st, entry));                
			}
		}
 	} 
 	
 	private String getSecurityString(TElZipArchiveDirectoryEntry entry) {
 		String st = "";
        if (entry.getEncrypted())
        {
            st = "Encrypted (";
            if (entry.getWinZipEncryption())
                st = st + "WinZip-AES" + entry.getWinZipAesKeySize() + ")";
            else if (entry.getStrongEncryption())
            {
                st = st + "Strong-";
                switch (entry.getStrongEncryptionInfo().getEncryptionAlgorithm())
                {
                    case SBZipUtils.ZIP_ENCRYPTION_DES: st = st + "DES"; break;
                    case SBZipUtils.ZIP_ENCRYPTION_RC2_OLD: st = st + "RC2"; break;
                    case SBZipUtils.ZIP_ENCRYPTION_RC2_NEW: st = st + "RC2"; break;
                    case SBZipUtils.ZIP_ENCRYPTION_RC4: st = st + "RC4"; break;
                    case SBZipUtils.ZIP_ENCRYPTION_3DES168: st = st + "3DES-168"; break;
                    case SBZipUtils.ZIP_ENCRYPTION_3DES112: st = st + "3DES-112"; break;
                    case SBZipUtils.ZIP_ENCRYPTION_AES128: st = st + "AES-128"; break;
                    case SBZipUtils.ZIP_ENCRYPTION_AES192: st = st + "AES-192"; break;
                    case SBZipUtils.ZIP_ENCRYPTION_AES256: st = st + "AES-256"; break;
                    case SBZipUtils.ZIP_ENCRYPTION_BLOWFISH: st = st + "Blowfish"; break;
                    case SBZipUtils.ZIP_ENCRYPTION_TWOFISH: st = st + "Twofish"; break;
                    default: st = st + "Unknown"; break;
                }

                st = st + "," + entry.getStrongEncryptionInfo().getKeyLength() + " bits)";

                if (entry.getSigned())
                    st = st + ", signed(" + entry.getSignatureCount() + ")";
            }
            else
                st = st + "old, weak)";
        }
        return st;
	}

	public String getColumnName(int column) {
        return columnNames[column];
    }

 	public boolean isCellEditable(int row, int col) {
        return false;
    }
 	
 	public Class<?> getColumnClass(int column) {
         switch (column) {
         	 case FILENAME_INDEX:
         	 case CALG_INDEX:
         	 case VERS_INDEX:
         	 case SEC_INDEX:
         		 return String.class;
             case CSIZE_INDEX:
             case SIZE_INDEX:
                return Integer.class;
             default:
                return Object.class;
         }
     }

     public Object getValueAt(int row, int col) {
         KeyDataRow record = (KeyDataRow)dataVector.get(row);
         switch (col) {
	         case FILENAME_INDEX:
	        	 return record.FileName;
             case CSIZE_INDEX:
                return record.CompressSize;
             case SIZE_INDEX:
                return record.Size;
             case CALG_INDEX:
                return record.Alg;
             case VERS_INDEX:
                 return record.Ver;
             case SEC_INDEX:
                 return record.Security;                 
             default:
                return new Object();
         }
     }

     public void setValueAt(Object value, int row, int column) {
    	 KeyDataRow record = (KeyDataRow)dataVector.get(row);
         switch (column) {
	         case FILENAME_INDEX:
	        	 record.FileName = (String)value;
	        	 break;
             case CSIZE_INDEX:
                record.CompressSize = (Integer)value;
                break;
             case SIZE_INDEX:
                record.Size = (Integer)value;
                break;
             case CALG_INDEX:
                record.Alg = (String)value;
                break;
             case VERS_INDEX:
                 record.Ver = (String)value;
                 break;
             case SEC_INDEX:
                 record.Security = (String)value;
                 break;                 
             default:
                System.out.println("invalid index");
         }
         fireTableCellUpdated(row, column);
     }

     public int getRowCount() {
         return dataVector.size();
     }

     public int getColumnCount() {
         return columnNames.length;
     }

	public ArrayList getSelectedItems(int[] indexes) {
		ArrayList result = new ArrayList();
		for(int i = 0; i < dataVector.size(); i++){
			if (selectedIndex(i, indexes))
			{
				KeyDataRow row = (KeyDataRow)dataVector.get(i);
				result.add((TElZipArchiveDirectoryEntry)row.Tag);
			}
		}
		return result;
	}

	public void removeSelectedItems(int[] indexes) {
		Vector<KeyDataRow> result = new Vector<KeyDataRow>();
		for(int i = 0; i < dataVector.size(); i++){
			if (selectedIndex(i, indexes))
				fireTableRowsDeleted(i, i);
			else{
				KeyDataRow row = (KeyDataRow)dataVector.get(i);
				result.add(row);				
			}
		}
		dataVector.clear();
		dataVector.addAll(result);
	}

	private boolean selectedIndex(int i, int[] indexes) {
		for (int j = 0; j < indexes.length; j++)
			if (i == indexes[j]) return true;
		return false;
	}

	public TElZipArchiveDirectoryEntry getFirstSelectedItem(int firstRow) {
		return (TElZipArchiveDirectoryEntry)((KeyDataRow)dataVector.get(firstRow)).Tag;
	}
 }